/**
 * Gregor Richter und Markus Schwenk 2008
 * 
 * uart.c dient zur Kommunikation ber die seriellen Schnittstelle
 *
 */
 
#include <avr/io.h>
#include <inttypes.h>
#include <util/delay.h>
#include <string.h>

#include "uart.h"

//Berechnung fr das Baud-Rate-Register
#define UBRR_BAUD   ((F_CPU/(16UL*BAUD))-1)

/**
 * USART initialisieren
 */
void uart_init(void)
{
    // Baudrate einstellen, Normaler Modus
    UBRRH = (UBRR_BAUD>>8);
    UBRRL = (UBRR_BAUD & 0x0ff);

    // Aktivieren von receiver und transmitter
    UCSRB = (1 << RXEN) | (1 << TXEN);

    // Einstellen des Datenformats: 8 Datenbits, 1 Stoppbit
    UCSRC = (1<<URSEL)|(1<<UCSZ1)|(1<<UCSZ0);
}

/**
 *
 * Gibt Einzelnes Zeichen an der seriellen Schnittstelle aus
 */
void uart_put_char(uint8_t temp)
{	
	//Warte bis Senderegiester frei ist
	while ( !(UCSRA & (1<<UDRE)) );
	//Sende den Wert temp
	_delay_us(964);
	UDR = temp;
	
}

/**
 *
 * Gibt eine Zeichenkette an der seriellen Schnitstelle aus
 */
void uart_put_string(char *string)
{
	while(*string)
	{
		uart_put_char(*string);
		string++;
	}
}

/**
 * gibt den Aktuellen Wert der seriellen Schnittstelle zurck oder
 * bei einem "TIMEOUT" 0
 */
uint8_t uart_get_char(void)
{	
	// !(UCSRA & (1 << RXC))==TRUE, wenn serielle Daten empfangen wurden
	for(uint16_t i = 0 ;!(UCSRA & (1 << RXC)); i++)
	{
		if(i > TIMEOUT)
			return 0; //Timeout: gebe 0 zurck
		
		//warte genau 1ms
		_delay_ms(1); 
	}
		//Gebe sonst Zeichen zurck!
		return UDR;
}

/**
 * vergleich *string mit der seriellen Schnittstelle
 * Diese routine Dient als Warteschleife, bis eine bestimmt
 * Zeichenfolge gesendet wurde
 * kommt es zu einem Timeout, wird 0 zurckgegeben
 */
uint8_t uart_wait_until(char *string)
{
	uint8_t temp;
	uint16_t temp16 = 0;
	for (uint8_t i = 0; i < strlen(string); temp16++ )
	{	
		//Speichere aktuellen Wert 
		temp = uart_get_char();
		
		// Prfe, ob es zu einem Timeout gekommen ist
		if (temp == 0 || temp16 > TIMEOUT) 
			return 0;	//Wenn ja, dann wird 0 zurckgegeben
		// wenn aktuelle Zeichen nicht bereinstimmt
		else if (string[i] != temp)	
		{
			// fang von Vorne an
			i = 0;
			// berspringe i++
			continue;
		}
		
		i++;
		
	}
	return 1;
}

